@@ -1,12 +1,10 @@
 # -*- coding: UTF-8 -*-
-# FIXME: really, fix the "variant" concept..
-# TODO: option to give ignored suffix for e.g. downloading..?
-# TODO: Work on SubTypeAwareDisplay (partially done; consider using "poorMansInference")
-#   - or <http://dev.w3.org/cvsweb/~checkout~/2004/PythonLib-IH/RDFSClosure.py>
-"""
+"""This module contains everything needed to setup an Oort-based web 
+application. It also imports relevant parts from ``oort.display``.
 """
 #=======================================================================
 import urllib
+import logging
 from pkg_resources import EntryPoint
 from rdflib import URIRef
 from paste.wsgiwrappers import WSGIRequest, WSGIResponse
@@ -17,46 +15,53 @@ from oort.rdfview import RdfQuery
 from oort.util.code import SlotStruct, contract, autosuper
 __metaclass__ = autosuper
 #=======================================================================
+# FIXME: really, fix the "variant" concept..
+# TODO: option to give ignored suffix for e.g. downloading..?
+# TODO: Work on SubTypeAwareDisplay (partially done; consider using "poorMansInference")
+#   - or <http://dev.w3.org/cvsweb/~checkout~/2004/PythonLib-IH/RDFSClosure.py>
 
 
 class resource_viewer_meta(autosuper):
     def __init__(cls, clsName, bases, clsDict):
         autosuper.__init__(cls, clsName, bases, clsDict)
-        displays = clsDict.setdefault('displays', {})
+        displayClasses = clsDict.setdefault('displayClasses', {})
         defaultDisplay = clsDict.get('defaultDisplay')
         for member in clsDict.values():
             if isinstance(member, type) and issubclass(member, DisplayBase):
                 name = getattr(member, 'name', None)
                 if not name:
                     continue
-                displays[name] = member()
+                displayClasses[name] = member
                 if not defaultDisplay and getattr(member, 'default', False):
                     defaultDisplay = name
-        cls.displays = displays
+        cls.displayClasses = displayClasses
         cls.defaultDisplay = defaultDisplay
 
 
 class ResourceViewerBase:
+    """Base class - not for direct usage."""
     __metaclass__ = resource_viewer_meta
 
     @classmethod
     def app_factory(cls, global_conf=None,
-                    graph_factory=None, debug='False', **kwargs):
+                    graph_factory=None, **cfg):
+        """Factory method compatible with the app_factory feature of Paste Deploy."""
         mk_graph = EntryPoint.parse("x=%s" % graph_factory).load(False)
         graph = mk_graph()
-        debug = debug.capitalize() == str(True)
-        return cls(graph, debug)
+        configure_logging(logging, cfg)
+        return cls(graph, cfg)
 
     VARIANT_SEP = '-'
     RESOURCE_QUERY_KEY = 'resource'
 
-    #displays = {}
     #defaultDisplay = None
     langOrder = ()
 
-    def __init__(self, graph, debug=False):
+    def __init__(self, graph, cfg=None):
         self.graph = graph
-        self._debug = debug
+        self._logger = logging.getLogger("oort.sitebase.ResourceViewerBase")
+        self.displays = dict((name, dispCls(cfg=cfg))
+                for name, dispCls in self.displayClasses.items())
 
     def __call__(self, environ, start_response):
         req = WSGIRequest(environ)
@@ -97,15 +102,17 @@ class ResourceViewerBase:
     def resource_to_app_path(self, resource):
         raise NotImplementedError
 
-    def app_url_for(self, lang, displayArg, resource=None):
+    def app_url_for(self, lang, displayArg, resource=None, fallback=True):
         urlBase = self._viewPathTemplate % (lang, displayArg)
         if resource:
             resource, success = self.resource_to_app_path(resource)
             if success:
                 return urlBase + resource.replace('#', '%23')
-            else:
+            elif fallback:
                 return '%s?%s=%s' % (urlBase, self.RESOURCE_QUERY_KEY,
                                      urllib.quote(resource))
+            else:
+                return None # TODO: raise NoAppUrlForResourceError(resource)
         else:
             return urlBase
 
@@ -116,8 +123,9 @@ class ResourceViewerBase:
         current = Current(resource, lang)
         appUtil = AppUtil(current, displayArg, self, req)
 
-        if self._debug:
-            print "DEBUG:", current, appUtil, display # TODO: use __repr__ and logging
+        self._logger.debug("Prepared display with: %s %s %s"
+                            % (current, appUtil, display ))
+        # TODO: use __repr__?
 
         def get_result(graph):
             return display.get_result_generator(graph, appUtil, variantArg)
@@ -125,7 +133,19 @@ class ResourceViewerBase:
         return get_result, display.contentType
 
 
+LOG_LEVEL_NAMES = 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'
+
+def configure_logging(logging, cfg):
+    logCfg = {}
+    level = cfg.get('logging_level', 'warning').upper()
+    #format, filename, filemode
+    if level in LOG_LEVEL_NAMES:
+        logCfg['level'] = getattr(logging, level)
+    logging.basicConfig(**logCfg)
+
+
 class Current(SlotStruct):
+    """Representation of the current *resource* request."""
     __slots__ = 'resource', 'lang'
 
 class AppUtil(SlotStruct):
@@ -133,24 +153,28 @@ class AppUtil(SlotStruct):
     'app'. It provides a controlled interface to the application and the 
     current state (request)."""
 
-    __slots__ = 'current', '_displayArg', '_view', '_req'
+    __slots__ = 'current', '_displayArg', '_viewer', '_req'
 
     def queries(self, queries):
         for query in queries:
-            yield query(self._view.graph, self.current.resource, self.current.lang)
+            yield query(self._viewer.graph, self.current.resource, self.current.lang)
 
-    def link(self, resource=None, lang=None, display=None, displayArg=None):
+    def link(self, resource=None, lang=None, display=None, fallback=True, displayArg=None):
         """Returns an URL within the current app. The following arguments 
         can be given (all defaults to current unless provided):
 
             resource
-                a target URI or an instance of an RdfQuery
+                A target URI or an instance of an RdfQuery.
 
             lang
-                the language to use
+                The language to use.
 
             display
-                the display name to use
+                The display name to use.
+
+            fallback
+                Revert to a GET for the resource if no app url can be made.
+                Defaults to True.
         """
         if resource:
             if isinstance(resource, RdfQuery):
@@ -158,7 +182,10 @@ class AppUtil(SlotStruct):
         else: resource = self.current.resource
         lang = lang or self.current.lang
         displayArg = display or displayArg or self._displayArg
-        return self._view.app_url_for(lang, displayArg, resource)
+        return self._viewer.app_url_for(lang, displayArg, resource, fallback)
+
+    def safe_link(self, *args):
+        return self.link(fallback=False)
 
     @property
     def baseurl(self):
@@ -167,6 +194,8 @@ class AppUtil(SlotStruct):
 
 
 class ResourceViewer(ResourceViewerBase):
+    """Use for simple apps where all URIs in the graph share the same base URI 
+    (set ``resourceBase`` to that)."""
     resourceBase =None
     trailSep='/'
 
@@ -186,6 +215,9 @@ class ResourceViewer(ResourceViewerBase):
 
 
 class MultiBaseResourceViewer(ResourceViewerBase):
+    """Use for full-fledged apps where many different base URIs will be 
+    "mounted" at different named bases (done with the dictionary 
+    ``resourceBases``)."""""
     resourceBases = {}
     defaultResource = ''
     # TODO: skip defaultResource and use defaultResourceBase?
